/******************************************************************************\
 gnofin/text-export.c   $Revision: 1.4 $
 Copyright (C) 1999 Darin Fisher

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\******************************************************************************/

#include "common.h"
#include <string.h>
#include <errno.h>
#include "data-if.h"
#include "dialogs.h"
#include "record-stringizer.h"
#include "text-export.h"

static const struct
{
  guint id;
  const gchar *name;
}
fields[] = 
{
  { RECORD_FIELD_DATE,           N_("Date")        },
  { RECORD_FIELD_TYPE,           N_("Type")        },
  { RECORD_FIELD_NUMBER,         N_("Number")      },
  { RECORD_FIELD_LINKED_ACCOUNT, N_("Transfer")    },
  { RECORD_FIELD_PAYEE,          N_("Payee/payor") },
  { RECORD_FIELD_MEMO,           N_("Memo")        },
  { RECORD_FIELD_CLEARED,        N_("S")           },
  { RECORD_FIELD_AMOUNT,         N_("Amount")      },
  { RECORD_FIELD_CLEARED_BAL,    N_("Cleared Bal") },
  { RECORD_FIELD_OVERALL_BAL,    N_("Overall Bal") },
};

gboolean
text_export (GtkWindow *parent, const gchar *filename, const Bankbook *book)
{
  FILE *file;
  const GList *ac, *rc;

  trace("");

  file = fopen (filename, "wt");
  if (file == NULL)
  {
    dialog_error (parent, _("Unable to write file: %s"), strerror (errno));
    return FALSE;
  }

  for (ac=if_bankbook_get_accounts (book); ac; ac=ac->next)
  {
    Account *account = LIST_DEREF (Account, ac);
    guint i = 0;

    fprintf (file, "%s\n", if_account_get_name (account));

    fprintf (file, _(fields[i].name));
    for (i=1; i<sizeof_array (fields); ++i)
      fprintf (file, "\t%s", _(fields[i].name));
    fprintf (file, "\n");

    for (rc=if_account_get_records (account); rc; rc=rc->next)
    {
      RecordInfo rec = {0};
      Record *record = LIST_DEREF (Record, rc);
      gchar *text;

      if_record_get_info (record, 0, &rec);

      for (i=0; i<sizeof_array (fields); ++i)
      {
        text = stringize_record_field (NULL, 0, fields[i].id, &rec);
        fprintf (file, text);
	if (i == (sizeof_array (fields) - 1))
	  fprintf (file, "\n");
	else
	  fprintf (file, "\t");
	g_free (text);
      }
    }
    if (ac->next)
      fprintf (file,"\n");
  }
  fclose (file);
  return TRUE;
}
