/******************************************************************************\
 gnofin/dialogs.h   $Revision: 1.6 $
 Copyright (C) 1999-2000 Darin Fisher

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\******************************************************************************/

#ifndef DIALOGS_H
#define DIALOGS_H

#include <gtk/gtkwindow.h>
#include <libgnomeui/gnome-messagebox.h>
#include "data-types.h"
#include "ui-types.h"

GNOFIN_API_BEGIN

/* simple modal dialogs */
void dialog_message (GtkWindow *parent, const gchar *message_type, const gchar *message, ... );
#define dialog_notice(w, m...)   dialog_message(w?GTK_WINDOW(w):w, GNOME_MESSAGE_BOX_INFO, ##m)
#define dialog_warning(w, m...)  dialog_message(w?GTK_WINDOW(w):w, GNOME_MESSAGE_BOX_WARNING, ##m)
#define dialog_error(w, m...)    dialog_message(w?GTK_WINDOW(w):w, GNOME_MESSAGE_BOX_ERROR, ##m)

/* modal question dialog can return 3 different values */
enum {
  DIALOG_YES,
  DIALOG_NO,
  DIALOG_CANCEL
};
gint dialog_question_yes_no (GtkWindow *parent, const gchar *message);
gint dialog_question_yes_no_cancel (GtkWindow *parent, const gchar *message);
#define dialog_question(p,m) dialog_question_yes_no_cancel (p,m)

typedef struct {
  GList *labels;     /* list of strings displayed in format selector */
  gint   selection;  /* -1 if no selection */
} FileFormatSelection;

/* modal wrapper to the standard GtkFileSelection widget */
gchar *dialog_query_filename (GtkWindow           *parent, 
			      const gchar         *title, 
			      const gchar         *default_filename,   // can be NULL
			      gint                 overwrite_protect,
			      FileFormatSelection *format_selection);  // can be NULL

/* the return value indicates which fields in the info structure were changed */
guint dialog_account_info (GtkWindow *parent, AccountInfo *info);

void dialog_about   (GtkWindow *parent);

typedef struct {
  RecordTypeInfo info;        /* type info to be modified */
  RecordType    *type;        /* type pointer used as an identifier (NULL if none) */
  guint          mask;        /* indicates which fields have been changed */
  guint          remove : 1;  /* TRUE if record type should be removed */
} RecordTypeContext;

/* return value indicates whether data has changed */
gboolean dialog_edit_record_types (GtkWindow *parent, GSList **contexts);

GNOFIN_API_END

#endif
