#ifndef GNOFIN_API_H
#define GNOFIN_API_H

/* Plugins can query the API version from Gnofin to determine if
 * it exports a compatible interface.  This allows 3rd party
 * plugins to exist on an independent development cycle.
 *
 * The major API version number indicates the data structure
 * alignments.  The minor API version number indicates the
 * possible addition of API functions.  Hence, if Gnofin reports 
 * its API version as 1.3, and a plugin was built against 1.1,
 * the plugin can be sure to work properly.  However, a plugin
 * built against say an API version of 1.4 can not be sure to
 * work properly as there may be undefined functions.
 *
 * The API in question is defined by all headers file enclosed by:
 * 
 *    GNOFIN_API_BEGIN
 *    ...
 *    
 *    GNOFIN_API_END
 *
 * Plugins should consider all other header files "private" and
 * hence subject to change.
 */

#ifdef __cplusplus
# define GNOFIN_API_BEGIN extern "C" {
# define GNOFIN_API_END }
#else
# define GNOFIN_API_BEGIN
# define GNOFIN_API_END
#endif

GNOFIN_API_BEGIN

#define GNOFIN_API_VERSION_MAJOR 2
#define GNOFIN_API_VERSION_MINOR 0

void gnofin_api_version_get (guint *major, guint *minor);

GNOFIN_API_END

#endif
