/******************************************************************************\
 gnofin/money.c   $Revision: 1.2 $
 Copyright (C) 1999-2000 Darin Fisher

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\******************************************************************************/

#include "common.h"
#include <stdlib.h>
#include <locale.h>
#include <stdio.h>
#include <math.h>
#include <glib.h>
#include "money.h"

/* FIXME: what about using the strfmon function in the standard library?
 *        unfortunately i can't find any documentation on strfmon and there
 *        also doesn't appear to be a corresponding strpmon! */

static struct lconv *lconv = NULL;

static char
get_thousands_sep ()
{
  if (lconv == NULL)
    lconv = localeconv ();

  return lconv->mon_thousands_sep[0];
}

gchar *
money_stringize (gchar *buf, int maxlen, money_t amount)
{
  double d;

  trace ("");

  d = ((double) amount) / 100.0;

  if (buf)
    snprintf(buf, maxlen, "%.02f", d);
  else
    buf = g_strdup_printf ("%.02f", d);

  return buf;
}

gboolean
money_parse (const gchar *buf, money_t *amount)
{
  double d;
  gchar thousep, *endptr;

  trace ("");
  g_return_val_if_fail (buf, FALSE);
  g_return_val_if_fail (amount, FALSE);

  if (buf[0] == '\0')
  {
    trace ("no chars to convert!");
    return FALSE;
  }

  thousep = get_thousands_sep ();
  if (thousep != '\0')
  {
    gchar *ptr = (gchar *)buf;
    gchar *savptr = (gchar *)buf;
    gchar ch;

    while (*buf)
    {
      ch = *buf++;
      if (ch == thousep)
        continue;
      *ptr++ = ch;
    }
    *ptr = '\0';
    buf = savptr;
  }

/*
  if (sscanf(buf, "%lf", &d) == 0)
    return FALSE;
 */
  d = g_strtod (buf, &endptr);

  if (endptr == buf)
  {
    trace ("no chars converted!");
    return FALSE;
  }
  if (strlen (endptr) > 0)
  {
    trace ("not all chars converted!");
    return FALSE;
  }

  *amount = (long) rint(d * 100.0);

  return TRUE;
}

gint
money_compare (money_t a, money_t b)
{
  a = money_sign (a);
  b = money_sign (b);

  /* Group positive amounts before negative amounts */

  if ((a > 0) && (b < 0))
    return -1;
  else if ((b > 0) && (a < 0))
    return +1;
  else
    return 0;
}

// vim: ts=8 sw=2
