/******************************************************************************\
 gnofin/stock-record-types.c   $Revision: 1.4 $
 Copyright (C) 1999-2000 Darin Fisher

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\******************************************************************************/

#include "common.h"
#include <libgnome/gnome-config.h>
#include "stock-record-types.h"
#include "data-if.h"
#include "config-saver.h"

typedef struct {
  guint           num_types;
  RecordTypeInfo *record_types;
} Config;

static const RecordTypeInfo default_record_types[] =
{
  { N_("ATM"),  N_("ATM Cash"),               0, 0, RECORD_TYPE_SIGN_NEG },
  { N_("CHK"),  N_("Check"),                  1, 0, RECORD_TYPE_SIGN_NEG },
  { N_("CC"),   N_("Check Card"),             0, 0, RECORD_TYPE_SIGN_NEG },
  { N_("DEP"),  N_("Deposit"),                0, 0, RECORD_TYPE_SIGN_POS },
  { N_("EFT"),  N_("Electronic Transaction"), 0, 0, RECORD_TYPE_SIGN_ANY },  
  { N_("FEE"),  N_("Bank Fee"),               0, 0, RECORD_TYPE_SIGN_NEG },
  { N_("TXFR"), N_("Transfer"),               0, 1, RECORD_TYPE_SIGN_ANY },
};

#define CAT  "DefaultRecordTypes"
#define KEY  "/" PACKAGE "/" CAT "/"

static void
load_config (Config *config)
{
  gchar path[512];
  guint i;

  trace ("");
  g_return_if_fail (config);

  g_snprintf (path, sizeof path, KEY "num_types=%d",
  	      sizeof_array (default_record_types));
  config->num_types = gnome_config_get_int (path);

  config->record_types = g_new0 (RecordTypeInfo, config->num_types);

  for (i = 0; i < config->num_types; ++i)
  {
    g_snprintf (path, sizeof path, KEY "record_type[%d].name=%s", i,
	        default_record_types[i].name);
    config->record_types[i].name = gnome_config_get_string (path);

    g_snprintf (path, sizeof path, KEY "record_type[%d].description=%s", i,
	        _(default_record_types[i].description));
    config->record_types[i].description = gnome_config_get_string (path);

    g_snprintf (path, sizeof path, KEY "record_type[%d].numbered=%d", i,
	        default_record_types[i].numbered);
    config->record_types[i].numbered = gnome_config_get_int (path);

    g_snprintf (path, sizeof path, KEY "record_type[%d].linked=%d", i,
	        default_record_types[i].linked);
    config->record_types[i].linked = gnome_config_get_int (path);

    g_snprintf (path, sizeof path, KEY "record_type[%d].sign=%d", i,
	        default_record_types[i].sign);
    config->record_types[i].sign = gnome_config_get_int (path);
  }
}

static void
save_config (const Config *config)
{
  gchar path[512];
  guint i;

  trace ("");
  g_return_if_fail (config);

  gnome_config_set_int (KEY "num_types", config->num_types);
  for (i=0; i<config->num_types; ++i)
  {
    g_snprintf (path, sizeof path, KEY "record_type[%d].name", i);
    gnome_config_set_string (path, config->record_types[i].name);

    g_snprintf (path, sizeof path, KEY "record_type[%d].description", i);
    gnome_config_set_string (path, config->record_types[i].description);

    g_snprintf (path, sizeof path, KEY "record_type[%d].numbered", i);
    gnome_config_set_int (path, config->record_types[i].numbered);

    g_snprintf (path, sizeof path, KEY "record_type[%d].linked", i);
    gnome_config_set_int (path, config->record_types[i].linked);

    g_snprintf (path, sizeof path, KEY "record_type[%d].sign", i);
    gnome_config_set_int (path, config->record_types[i].sign);
  }
}

static Config *
get_config (void)
{
  static Config config = {0};
  static gboolean init = FALSE;

  if (!init)
  {
    load_config (&config);
    config_saver_register (CAT, (ConfigSaveFunc) save_config, &config);
    init = TRUE;
  }
  return &config;
}

void
install_stock_record_types (Bankbook *book)
{
  Config *config = get_config ();
  guint i;

  trace ("");
  g_return_if_fail (book);

  for (i = 0; i < config->num_types; ++i)
    if_bankbook_insert_record_type (book, &config->record_types[i]);
}
