/******************************************************************************\
 gnofin/ui-menubar.c   $Revision: 1.9 $
 Copyright (C) 1999-2000 Darin Fisher

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
\******************************************************************************/

//#define ENABLE_DEBUG_TRACE

#include "common.h"
#include <gtk/gtkmenushell.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtktearoffmenuitem.h>
#include <gdk/gdkkeysyms.h>
#include <libgnomeui/gnome-app.h>
#include <libgnomeui/gnome-app-helper.h>
#include <libgnomeui/gnome-stock.h>
#include "ui.h"
#include "ui-menubar.h"
#include "ui-statusbar.h"
#include "data-if.h"
#include "dialogs.h"
#include "file-filter.h"
#include "file-mru.h"
#include "preferences.h"


static const gchar *file_filter_key = "ui-menubar-file-filter";


/******************************************************************************
 * Menu handlers
 */

static void
on_nop (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  dialog_notice (ui_get_window (menubar->ui),
	       _("Sorry, but this feature is not yet implemented."));
}

static void
on_file_new (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_load_bankbook (menubar->ui, NULL, NULL);
}

static void
on_file_open (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_load_file (menubar->ui, NULL);
}

static void
on_file_save (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_save_file (menubar->ui, ui_get_current_filename (menubar->ui));
}

static void
on_file_save_as (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_save_file (menubar->ui, NULL);
}

static void
on_file_exit (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  ui_exit (menubar->ui);
}

static void
on_edit_undo (GtkWidget *w, UI_Menubar *menubar)
{
  Bankbook *book;

  trace ("");
  g_return_if_fail (menubar);

  book = ui_get_current_bankbook (menubar->ui);
  if (book)
    if_bankbook_undo (book);
}

static void
on_edit_redo (GtkWidget *w, UI_Menubar *menubar)
{
  Bankbook *book;

  trace ("");
  g_return_if_fail (menubar);

  book = ui_get_current_bankbook (menubar->ui);
  if (book)
    if_bankbook_redo (book);
}

static void
on_edit_delete (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_remove_current_record (menubar->ui);
}

static void
on_edit_toggle_status (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_toggle_current_record_status (menubar->ui);
}

static void
on_edit_define_record_types (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_modify_current_record_types (menubar->ui);
}

static void
on_edit_preferences (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);
  g_return_if_fail (menubar->ui);

  preferences_dialog_run (ui_get_window (menubar->ui));
}

static void
on_account_insert (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_insert_account (menubar->ui);
}

static void
on_account_delete (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_remove_current_account (menubar->ui);
}

static void
on_account_props (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);

  ui_modify_current_account (menubar->ui);
}

static void
on_help_about (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");
  g_return_if_fail (menubar);
  g_return_if_fail (menubar->ui);

  dialog_about (ui_get_window (menubar->ui));
}

#ifdef ENABLE_DUMP_STATE
static void
on_dump_state (GtkWidget *w, UI_Menubar *menubar)
{
  trace ("");

  if (menubar && menubar->ui)
  {
    Bankbook *book = ui_get_current_bankbook (menubar->ui);

    if (book)
      if_bankbook_dump (book);
  }
}
#endif

static void
on_file_import (GtkMenuItem *item, UI_Menubar *menubar)
{
  GtkWindow *win;
  FileFilter *filt;
  gchar *fname;

  trace ("");
  g_return_if_fail (menubar);
  g_return_if_fail (menubar->ui);

  filt = (FileFilter *) gtk_object_get_data (GTK_OBJECT (item), file_filter_key);
  g_assert (filt);

  win = ui_get_window (menubar->ui);
  fname = dialog_query_filename (win, _("Specify input file"), NULL, FALSE, NULL);

  if (fname)
  {
    Bankbook *book;

    book = if_bankbook_new ();
    if (filt->import (win, fname, book))
      ui_load_bankbook (menubar->ui, book, NULL);
    else
      if_bankbook_destroy (book);

    g_free (fname);
  }
}

static void
on_file_export (GtkMenuItem *item, UI_Menubar *menubar)
{
  GtkWindow *win;
  FileFilter *filt;
  gchar *fname;

  trace ("");

  filt = (FileFilter *) gtk_object_get_data (GTK_OBJECT (item), file_filter_key);
  g_assert (filt);

  win = ui_get_window (menubar->ui);
  fname = dialog_query_filename (win, _("Specify output file"), NULL, FALSE, NULL);

  if (fname)
  {
    filt->export (win, fname, ui_get_current_bankbook (menubar->ui));
    g_free (fname);
  }
}

static void
on_file_open_recent (GtkMenuItem *item, UI_Menubar *menubar)
{
  GtkWidget *label;
  gchar *text;

  trace ("");

  label = GTK_BIN (item)->child;
  gtk_label_get (GTK_LABEL (label), &text);

  ui_load_file (menubar->ui, text);
}

static void
on_import_select (GtkMenuItem *item, UI_Menubar *menubar)
{
  FileFilter *filt;
  gchar *buf;
  trace ("");

  filt = (FileFilter *) gtk_object_get_data (GTK_OBJECT (item), file_filter_key);
  g_assert (filt);

  buf = g_strdup_printf ("Import \"%s\" file format", filt->label);
  gnome_appbar_push (GNOME_APPBAR (menubar->ui->statusbar->appbar), buf);
  g_free (buf);
}

static void
on_export_select (GtkMenuItem *item, UI_Menubar *menubar)
{
  FileFilter *filt;
  gchar *buf;
  trace ("");

  filt = (FileFilter *) gtk_object_get_data (GTK_OBJECT (item), file_filter_key);
  g_assert (filt);

  buf = g_strdup_printf ("Export \"%s\" file format", filt->label);
  gnome_appbar_push (GNOME_APPBAR (menubar->ui->statusbar->appbar), buf);
  g_free (buf);
}

static void
on_file_recent_select (GtkMenuItem *item, UI_Menubar *menubar)
{
  gchar *buf, *text;

  trace ("");

  gtk_label_get (GTK_LABEL (GTK_BIN (item)->child), &text);

  buf = g_strdup_printf ("Open \"%s\"", text);
  gnome_appbar_push (GNOME_APPBAR (menubar->ui->statusbar->appbar), buf);
  g_free (buf);
}

static void
on_menuitem_deselect (GtkMenuItem *item, UI_Menubar *menubar)
{
  trace ("");
  gnome_appbar_pop (GNOME_APPBAR (menubar->ui->statusbar->appbar));
}

static void
submenu_clear (GtkMenu *submenu)
{
  /* This function removes any menu items from a submenu.  It is careful
   * however not to remove any tearoff menu items. */

  GList *children = gtk_container_children (GTK_CONTAINER (submenu));
  while (children)
  {
    GtkWidget *child = GTK_WIDGET (children->data);
    children = children->next;

    if (GTK_OBJECT (child)->klass->type != gtk_tearoff_menu_item_get_type ())
      gtk_container_remove (GTK_CONTAINER (submenu), child);
  }
}

static void
on_popup_imports (GtkMenuItem *item, UI_Menubar *menubar)
{
  trace ("");

  if (file_import_filters_changed (menubar->imp_stamp))
  {
    GtkMenu *submenu;
    FileFilter *filt;
    GSList *ff;

    trace ("rebuilding imports list");

    submenu = GTK_MENU (item->submenu);
    submenu_clear (submenu);

    ff = file_import_filters_get (&menubar->imp_stamp);
    for (; ff; ff=ff->next)
    {
      filt = LIST_DEREF (FileFilter, ff);
      trace ("filt=%p, filt->label=%s", filt, filt->label);

      item = GTK_MENU_ITEM (gtk_menu_item_new_with_label (filt->label));
      gtk_object_set_data (GTK_OBJECT (item), file_filter_key, filt);
      gtk_signal_connect (GTK_OBJECT (item), "activate",
      			  GTK_SIGNAL_FUNC (on_file_import), menubar);
      gtk_signal_connect (GTK_OBJECT (item), "select",
      			  GTK_SIGNAL_FUNC (on_import_select), menubar);
      gtk_signal_connect (GTK_OBJECT (item), "deselect",
      			  GTK_SIGNAL_FUNC (on_menuitem_deselect), menubar);
      gtk_widget_show (GTK_WIDGET (item));
      gtk_menu_append (submenu, GTK_WIDGET (item));
    }
  }
}

static void
on_popup_exports (GtkMenuItem *item, UI_Menubar *menubar)
{
  trace ("");

  if (file_export_filters_changed (menubar->exp_stamp))
  {
    GtkMenu *submenu;
    FileFilter *filt;
    GSList *ff;

    trace ("rebuilding exports list");

    submenu = GTK_MENU (item->submenu);
    submenu_clear (submenu);

    ff = file_export_filters_get (&menubar->exp_stamp);
    for (; ff; ff=ff->next)
    {
      filt = LIST_DEREF (FileFilter, ff);
      trace ("filt=%p, filt->label=%s", filt, filt->label);

      item = GTK_MENU_ITEM (gtk_menu_item_new_with_label (filt->label));
      gtk_object_set_data (GTK_OBJECT (item), file_filter_key, filt);
      gtk_signal_connect (GTK_OBJECT (item), "activate",
      			  GTK_SIGNAL_FUNC (on_file_export), menubar);
      gtk_signal_connect (GTK_OBJECT (item), "select",
      			  GTK_SIGNAL_FUNC (on_export_select), menubar);
      gtk_signal_connect (GTK_OBJECT (item), "deselect",
      			  GTK_SIGNAL_FUNC (on_menuitem_deselect), menubar);
      gtk_widget_show (GTK_WIDGET (item));
      gtk_menu_append (submenu, GTK_WIDGET (item));
    }
  }
}

static void
on_popup_file_mru (GtkMenuItem *item, UI_Menubar *menubar)
{
  trace ("");

  if (file_mru_changed (menubar->mru_stamp))
  {
    GtkMenu *submenu;
    const gchar *text;
    GSList *node;

    trace ("rebuilding recent files list");

    submenu = GTK_MENU (item->submenu);
    submenu_clear (submenu);

    node = file_mru_get (&menubar->mru_stamp);
    for (; node; node=node->next)
    {
      text = LIST_DEREF (const gchar, node);
      trace ("file=%s", text);

      item = GTK_MENU_ITEM (gtk_menu_item_new_with_label (text));
      gtk_signal_connect (GTK_OBJECT (item), "activate",
      			  GTK_SIGNAL_FUNC (on_file_open_recent), menubar);
      gtk_signal_connect (GTK_OBJECT (item), "select",
      			  GTK_SIGNAL_FUNC (on_file_recent_select), menubar);
      gtk_signal_connect (GTK_OBJECT (item), "deselect",
      			  GTK_SIGNAL_FUNC (on_menuitem_deselect), menubar);
      gtk_widget_show (GTK_WIDGET (item));
      gtk_menu_append (submenu, GTK_WIDGET (item));
    }
  }
}


/******************************************************************************
 * Menubar interface
 */

UI_Menubar *
ui_menubar_create (UI *ui)
{
  UI_Menubar * menubar;

  trace ("");
  g_return_val_if_fail (ui, NULL);
  g_return_val_if_fail (ui->statusbar, NULL);
  g_return_val_if_fail (ui->statusbar->appbar, NULL);

  menubar = g_new0 (UI_Menubar, 1);
  menubar->ui = ui;

  /* Create the menus */
  {
    GnomeUIInfo imports_menu[] =
    {
      GNOMEUIINFO_END
    };
    GnomeUIInfo exports_menu[] =
    {
      GNOMEUIINFO_END
    };
    GnomeUIInfo recent_files_menu[] =
    {
      GNOMEUIINFO_END
    };
    GnomeUIInfo file_menu[] =
    {
      GNOMEUIINFO_MENU_NEW_ITEM (N_("_New"), N_("Begin a new accounts file"), on_file_new, menubar),
      GNOMEUIINFO_MENU_OPEN_ITEM (on_file_open, menubar),
      GNOMEUIINFO_MENU_SAVE_ITEM (on_file_save, menubar),
      GNOMEUIINFO_MENU_SAVE_AS_ITEM (on_file_save_as, menubar),
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_SUBTREE (N_("_Import"), imports_menu),
      GNOMEUIINFO_SUBTREE (N_("_Export"), exports_menu),
 //   GNOMEUIINFO_SEPARATOR,
 //   { GNOME_APP_UI_ITEM, N_("_Plugins..."), N_("Run plugin manager"),
 //     on_nop, menubar, NULL,
 //     GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_SUBTREE (N_("_Recent Files"), recent_files_menu),
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_MENU_EXIT_ITEM (on_file_exit, menubar),
      GNOMEUIINFO_END
    };
    enum file_indices
    {
      file_new_idx,
      file_open_idx,
      file_save_idx,
      file_save_as_idx,
      file_imports_idx=5,
      file_exports_idx,
      file_mru_idx=8 //10
    };
    GnomeUIInfo edit_menu[] = 
    {
      GNOMEUIINFO_MENU_UNDO_ITEM (on_edit_undo, menubar),
      GNOMEUIINFO_MENU_REDO_ITEM (on_edit_redo, menubar),
      GNOMEUIINFO_SEPARATOR,
      { GNOME_APP_UI_ITEM, N_("_Delete"), N_("Delete selected record"),
	on_edit_delete, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, GDK_x, GDK_CONTROL_MASK, NULL },
      { GNOME_APP_UI_ITEM, N_("Toggle _Status"), N_("Toggle cleared status of selected record"),
	on_edit_toggle_status, menubar, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, GDK_c, GDK_CONTROL_MASK, NULL },
      GNOMEUIINFO_SEPARATOR,
      { GNOME_APP_UI_ITEM, N_("Define Record _Types..."), N_("Define record types"),
	on_edit_define_record_types, menubar, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
      GNOMEUIINFO_SEPARATOR,
      GNOMEUIINFO_MENU_PREFERENCES_ITEM (on_edit_preferences, menubar),
      GNOMEUIINFO_END
    };
    enum edit_indices
    {
      edit_undo_idx,
      edit_redo_idx,
      edit_delete_idx = 3,
      edit_toggle_status_idx,
    };
    GnomeUIInfo account_menu[] =
    {
      { GNOME_APP_UI_ITEM, N_("_Insert..."), N_("Insert a new account"),
	on_account_insert, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_NEW, 0, 0, NULL },
      { GNOME_APP_UI_ITEM, N_("_Delete"), N_("Delete this account"),
	on_account_delete, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE, 0, 0, NULL },
      GNOMEUIINFO_SEPARATOR,
      { GNOME_APP_UI_ITEM, N_("_Properties..."), N_("View/edit the properties of this account"),
	on_account_props, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 0, 0, NULL },
      GNOMEUIINFO_END
    };
    enum account_indices
    {
      account_insert_idx,
      account_delete_idx,
      account_props_idx = 3
    };
    GnomeUIInfo tools_menu[] =
    {
      { GNOME_APP_UI_ITEM, N_("_Find..."), N_("Find record"),
	on_nop, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, 0, NULL },
      { GNOME_APP_UI_ITEM, N_("_Reconcile..."), N_("Reconcile account data with bank statement"),
	on_nop, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, 0, NULL },
      { GNOME_APP_UI_ITEM, N_("_Reminders..."), N_("View/create/edit gnofin reminders"),
	on_nop, menubar, NULL,
	GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_EXEC, 0, 0, NULL },
      GNOMEUIINFO_END
    };
    GnomeUIInfo help_menu[] =
    {
      GNOMEUIINFO_MENU_ABOUT_ITEM (on_help_about, menubar),
#ifdef ENABLE_DUMP_STATE
      GNOMEUIINFO_SEPARATOR,
      { GNOME_APP_UI_ITEM, N_("Dump _State"), N_("Dump program state for help with debugging"),
	on_dump_state, menubar, NULL,
	GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL },
#endif
      GNOMEUIINFO_END
    };
    GnomeUIInfo main_menu[] =
    {
      GNOMEUIINFO_MENU_FILE_TREE (file_menu),
      GNOMEUIINFO_MENU_EDIT_TREE (edit_menu),
      GNOMEUIINFO_SUBTREE (N_("_Account"), account_menu),
      GNOMEUIINFO_SUBTREE (N_("_Tools"), tools_menu),
      GNOMEUIINFO_MENU_HELP_TREE (help_menu),
      GNOMEUIINFO_END
    };

    const struct
    {
      GtkWidget **ptr;
      GnomeUIInfo *tree;
      gint index;
    }
    menus[] =
    {
      { &menubar->file_save,          file_menu,      file_save_idx },
      { &menubar->file_save_as,       file_menu,      file_save_as_idx },
      { &menubar->file_imports,       file_menu,      file_imports_idx },
      { &menubar->file_exports,       file_menu,      file_exports_idx },
      { &menubar->file_mru,           file_menu,      file_mru_idx },
      { &menubar->edit_undo,          edit_menu,      edit_undo_idx },
      { &menubar->edit_redo,          edit_menu,      edit_redo_idx },
      { &menubar->edit_toggle_status, edit_menu,      edit_toggle_status_idx },
      { &menubar->edit_delete,        edit_menu,      edit_delete_idx },
      { &menubar->account_delete,     account_menu,   account_delete_idx },
      { &menubar->account_props,      account_menu,   account_props_idx }
    };
    gint i;

    /* Remember and create the main menu */
    gnome_app_create_menus (GNOME_APP (ui->app), main_menu);
    gnome_app_install_appbar_menu_hints (ui->statusbar->appbar, main_menu);

    /* Remember some of the menu items, so we can enable/disable them later */
    for (i=0; i<sizeof_array(menus); ++i)
      *menus[i].ptr = menus[i].tree[menus[i].index].widget;

    gtk_signal_connect (GTK_OBJECT (menubar->file_imports), "select",
    			GTK_SIGNAL_FUNC (on_popup_imports), menubar);
    gtk_signal_connect (GTK_OBJECT (menubar->file_exports), "select",
    			GTK_SIGNAL_FUNC (on_popup_exports), menubar);
    gtk_signal_connect (GTK_OBJECT (menubar->file_mru), "select",
    			GTK_SIGNAL_FUNC (on_popup_file_mru), menubar);
  }
  return menubar;
}

void
ui_menubar_enable_save (UI_Menubar *menubar, gboolean enable)
{
  trace ("");
  g_return_if_fail (menubar);

  gtk_widget_set_sensitive (menubar->file_save, enable);
}

void
ui_menubar_enable_undo (UI_Menubar *menubar, gboolean enable)
{
  trace ("");
  g_return_if_fail (menubar);

  gtk_widget_set_sensitive (menubar->edit_undo, enable);
}

void
ui_menubar_enable_redo (UI_Menubar *menubar, gboolean enable)
{
  trace ("");
  g_return_if_fail (menubar);

  gtk_widget_set_sensitive (menubar->edit_redo, enable);
}

void
ui_menubar_enable_account_ops (UI_Menubar *menubar, gboolean enable)
{
  trace ("");
  g_return_if_fail (menubar);

  gtk_widget_set_sensitive (menubar->account_delete, enable);
  gtk_widget_set_sensitive (menubar->account_props, enable);
}

void
ui_menubar_enable_record_ops (UI_Menubar *menubar, gboolean enable)
{
  trace ("");
  g_return_if_fail (menubar);

  gtk_widget_set_sensitive (menubar->edit_toggle_status, enable);
  gtk_widget_set_sensitive (menubar->edit_delete, enable);
}
